@@ -46,11 +46,11 @@
 #include "textfile.h"
 #include "uinput.h"
 
+#include "../src/adapter.h"
+#include "../src/device.h"
 #include "../src/storage.h"
 #include "../src/manager.h"
 #include "../src/dbus-common.h"
-#include "adapter.h"
-#include "../src/device.h"
 
 #include "device.h"
 #include "error.h"
@@ -275,8 +275,9 @@ static gboolean rfcomm_io_cb(GIOChannel *chan, GIOCondition cond, gpointer data)
 	struct fake_input *fake = data;
 	const char *ok = "\r\nOK\r\n";
 	char buf[BUF_SIZE];
-	gsize bread = 0, bwritten;
+	ssize_t bread = 0, bwritten;
 	uint16_t key;
+	int fd;
 
 	if (cond & G_IO_NVAL)
 		return FALSE;
@@ -286,16 +287,19 @@ static gboolean rfcomm_io_cb(GIOChannel *chan, GIOCondition cond, gpointer data)
 		goto failed;
 	}
 
+	fd = g_io_channel_unix_get_fd(chan);
+
 	memset(buf, 0, BUF_SIZE);
-	if (g_io_channel_read(chan, buf, sizeof(buf) - 1,
-				&bread) != G_IO_ERROR_NONE) {
+	bread = read(fd, buf, sizeof(buf) - 1);
+	if (bread < 0) {
 		error("IO Channel read error");
 		goto failed;
 	}
 
 	DBG("Received: %s", buf);
 
-	if (g_io_channel_write(chan, ok, 6, &bwritten) != G_IO_ERROR_NONE) {
+	bwritten = write(fd, ok, 6);
+	if (bwritten < 0) {
 		error("IO Channel write error");
 		goto failed;
 	}
@@ -315,32 +319,6 @@ failed:
 	return FALSE;
 }
 
-static inline DBusMessage *not_supported(DBusMessage *msg)
-{
-	return g_dbus_create_error(msg, ERROR_INTERFACE ".Failed",
-							"Not supported");
-}
-
-static inline DBusMessage *in_progress(DBusMessage *msg)
-{
-	return g_dbus_create_error(msg, ERROR_INTERFACE ".InProgress",
-				"Device connection already in progress");
-}
-
-static inline DBusMessage *already_connected(DBusMessage *msg)
-{
-	return g_dbus_create_error(msg, ERROR_INTERFACE ".AlreadyConnected",
-					"Already connected to a device");
-}
-
-static inline DBusMessage *connection_attempt_failed(DBusMessage *msg,
-							const char *err)
-{
-	return g_dbus_create_error(msg,
-				ERROR_INTERFACE ".ConnectionAttemptFailed",
-				"%s", err ? err : "Connection attempt failed");
-}
-
 static void rfcomm_connect_cb(GIOChannel *chan, GError *err, gpointer user_data)
 {
 	struct input_conn *iconn = user_data;
@@ -349,8 +327,7 @@ static void rfcomm_connect_cb(GIOChannel *chan, GError *err, gpointer user_data)
 	DBusMessage *reply;
 
 	if (err) {
-		reply = connection_attempt_failed(iconn->pending_connect,
-								err->message);
+		reply = btd_error_failed(iconn->pending_connect, err->message);
 		goto failed;
 	}
 
@@ -363,7 +340,7 @@ static void rfcomm_connect_cb(GIOChannel *chan, GError *err, gpointer user_data)
 	fake->uinput = uinput_create(idev->name);
 	if (fake->uinput < 0) {
 		g_io_channel_shutdown(chan, TRUE, NULL);
-		reply = connection_attempt_failed(iconn->pending_connect,
+		reply = btd_error_failed(iconn->pending_connect,
 							strerror(errno));
 		goto failed;
 	}
@@ -772,7 +749,7 @@ static int disconnect(struct input_device *idev, uint32_t flags)
 	}
 
 	if (!iconn)
-		return ENOTCONN;
+		return -ENOTCONN;
 
 	return connection_disconnect(iconn, flags);
 }
@@ -852,7 +829,7 @@ static void interrupt_connect_cb(GIOChannel *chan, GError *conn_err,
 
 failed:
 	error("%s", err_msg);
-	reply = connection_attempt_failed(iconn->pending_connect, err_msg);
+	reply = btd_error_failed(iconn->pending_connect, err_msg);
 	g_dbus_send_message(idev->conn, reply);
 
 	if (iconn->ctrl_io)
@@ -877,8 +854,8 @@ static void control_connect_cb(GIOChannel *chan, GError *conn_err,
 
 	if (conn_err) {
 		error("%s", conn_err->message);
-		reply = connection_attempt_failed(iconn->pending_connect,
-							conn_err->message);
+		reply = btd_error_failed(iconn->pending_connect,
+						conn_err->message);
 		goto failed;
 	}
 
@@ -892,7 +869,7 @@ static void control_connect_cb(GIOChannel *chan, GError *conn_err,
 				BT_IO_OPT_INVALID);
 	if (!io) {
 		error("%s", err->message);
-		reply = connection_attempt_failed(iconn->pending_connect,
+		reply = btd_error_failed(iconn->pending_connect,
 							err->message);
 		g_error_free(err);
 		g_io_channel_shutdown(chan, TRUE, NULL);
@@ -943,13 +920,13 @@ static DBusMessage *input_device_connect(DBusConnection *conn,
 
 	iconn = find_connection(idev->connections, "HID");
 	if (!iconn)
-		return not_supported(msg);
+		return btd_error_not_supported(msg);
 
 	if (iconn->pending_connect)
-		return in_progress(msg);
+		return btd_error_in_progress(msg);
 
 	if (is_connected(iconn))
-		return already_connected(msg);
+		return btd_error_already_connected(msg);
 
 	iconn->pending_connect = dbus_message_ref(msg);
 	fake = iconn->fake;
@@ -978,17 +955,11 @@ static DBusMessage *input_device_connect(DBusConnection *conn,
 	error("%s", err->message);
 	dbus_message_unref(iconn->pending_connect);
 	iconn->pending_connect = NULL;
-	reply = connection_attempt_failed(msg, err->message);
+	reply = btd_error_failed(msg, err->message);
 	g_error_free(err);
 	return reply;
 }
 
-static DBusMessage *create_errno_message(DBusMessage *msg, int err)
-{
-	return g_dbus_create_error(msg, ERROR_INTERFACE ".Failed",
-							"%s", strerror(err));
-}
-
 static DBusMessage *input_device_disconnect(DBusConnection *conn,
 						DBusMessage *msg, void *data)
 {
@@ -997,7 +968,7 @@ static DBusMessage *input_device_disconnect(DBusConnection *conn,
 
 	err = disconnect(idev, 0);
 	if (err < 0)
-		return create_errno_message(msg, -err);
+		return btd_error_failed(msg, strerror(-err));
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
